@@ -2,4 +2,5 @@ from .flux import *  # noqa
 from .uncertainty import *  # noqa
 from .location import *  # noqa
 from .width import *  # noqa
-from .template_comparison import *
+from .template_comparison import *  # noqa
+from .correlation import *  # noqa
