@@ -2,15 +2,17 @@
 A module for analysis tools focused on determining fluxes of spectral features.
 """
 
-from functools import wraps
 import warnings
+from functools import wraps
 
 import numpy as np
+from astropy.nddata import StdDevUncertainty, VarianceUncertainty, InverseVariance
+
 from .. import conf
-from ..manipulation import extract_region
+from ..spectra import Spectrum1D
+from ..manipulation import extract_region, LinearInterpolatedResampler
 from .utils import computation_wrapper
 import astropy.units as u
-from astropy.stats import sigma_clip
 from astropy.stats import mad_std
 from astropy.utils.exceptions import AstropyUserWarning
 
@@ -19,7 +21,8 @@ __all__ = ['line_flux', 'equivalent_width', 'is_continuum_below_threshold',
            'warn_continuum_below_threshold']
 
 
-def line_flux(spectrum, regions=None):
+def line_flux(spectrum, regions=None,
+              mask_interpolation=LinearInterpolatedResampler):
     """
     Computes the integrated flux in a spectrum or region of a spectrum.
 
@@ -35,6 +38,12 @@ def line_flux(spectrum, regions=None):
         Region within the spectrum to calculate the gaussian sigma width. If
         regions is `None`, computation is performed over entire spectrum.
 
+    mask_interpolation : ``None`` or `~specutils.manipulation.LinearInterpolatedResampler`
+        Interpolator class used to fill up the gaps in the spectrum's flux
+        array, when the spectrum mask is not None. If set to ``None``, the
+        masked spectral bins are excised from the data without interpolation
+        and the bin edges of the adjacent bins are extended to fill the gap.
+
     Returns
     -------
     flux : `~astropy.units.Quantity`
@@ -46,10 +55,12 @@ def line_flux(spectrum, regions=None):
     While the flux can be computed on any spectrum or region, it should be
     continuum-subtracted to compute actual line fluxes.
     """
-    return computation_wrapper(_compute_line_flux, spectrum, regions)
+    return computation_wrapper(_compute_line_flux, spectrum, regions,
+                               mask_interpolation=mask_interpolation)
 
 
-def equivalent_width(spectrum, continuum=1, regions=None):
+def equivalent_width(spectrum, continuum=1, regions=None,
+                     mask_interpolation=LinearInterpolatedResampler):
     """
     Computes the equivalent width of a region of the spectrum.
 
@@ -71,6 +82,13 @@ def equivalent_width(spectrum, continuum=1, regions=None):
         will be assumed, otherwise units are required and must be the same as
         the ``spectrum.flux``.
 
+    mask_interpolation : ``None`` or `~specutils.manipulation.LinearInterpolatedResampler`
+        Interpolator class used to fill up the gaps in the spectrum's flux
+        array after an excise operation to ensure the mask shape can always be
+        applied when the spectrum mask is not None. If set to ``None``, the
+        masked spectral bins are excised from the data without interpolation
+        and the bin edges of the adjacent bins are extended to fill the gap.
+
     Returns
     -------
     ew : `~astropy.units.Quantity`
@@ -86,41 +104,82 @@ def equivalent_width(spectrum, continuum=1, regions=None):
     """
 
     kwargs = dict(continuum=continuum)
-    return computation_wrapper(_compute_equivalent_width, spectrum, regions, **kwargs)
+    return computation_wrapper(_compute_equivalent_width, spectrum, regions,
+                               mask_interpolation=mask_interpolation, **kwargs)
 
 
-def _compute_line_flux(spectrum, regions=None):
+def _compute_line_flux(spectrum, regions=None,
+                       mask_interpolation=LinearInterpolatedResampler):
 
     if regions is not None:
         calc_spectrum = extract_region(spectrum, regions)
     else:
         calc_spectrum = spectrum
 
-    # Average dispersion in the line region
-    avg_dx = np.abs(np.diff(calc_spectrum.spectral_axis.quantity))
+    # Account for the existence of a mask.
+    if hasattr(calc_spectrum, 'mask') and calc_spectrum.mask is not None:
+        mask = calc_spectrum.mask
+        new_spec = Spectrum1D(flux=calc_spectrum.flux[~mask],
+                              spectral_axis=calc_spectrum.spectral_axis[~mask])
+        if mask_interpolation is None:
+            return _compute_line_flux(new_spec)
+        else:
+            interpolator = mask_interpolation(extrapolation_treatment='zero_fill')
+            sp = interpolator(new_spec, calc_spectrum.spectral_axis)
+            flux = sp.flux
+    else:
+        flux = calc_spectrum.flux
 
-    line_flux = np.sum(calc_spectrum.flux[1:] * avg_dx)
+    dx = (np.abs(np.diff(calc_spectrum.spectral_axis.bin_edges)))
+    line_flux = np.sum(flux * dx)
+
+    line_flux.uncertainty = None
+
+    if calc_spectrum.uncertainty is not None:
+        # Can't handle masks via interpolation here, since interpolators
+        # only work with the flux array.
+        if isinstance(calc_spectrum.uncertainty, StdDevUncertainty):
+            variance_q = calc_spectrum.uncertainty.quantity ** 2
+        elif isinstance(calc_spectrum.uncertainty, VarianceUncertainty):
+            variance_q = calc_spectrum.uncertainty.quantity
+        elif isinstance(calc_spectrum.uncertainty, InverseVariance):
+            variance_q = 1/calc_spectrum.uncertainty.quantity
+        else:
+            message = ('Uncertainty type "{}" was not recognized by line_flux.  '
+                       'Proceeding without uncertainty in result.').format(calc_spectrum.uncertainty.uncertainty_type)
+            warnings.warn(message, AstropyUserWarning)
+            variance_q = None
+
+        if variance_q is not None:
+            line_flux.uncertainty = np.sqrt(
+                np.sum(variance_q * dx**2))
 
     # TODO: we may want to consider converting to erg / cm^2 / sec by default
     return line_flux
 
 
-def _compute_equivalent_width(spectrum, continuum=1, regions=None):
-
+def _compute_equivalent_width(spectrum, continuum=1, regions=None,
+                              mask_interpolation=LinearInterpolatedResampler):
     if regions is not None:
         calc_spectrum = extract_region(spectrum, regions)
     else:
         calc_spectrum = spectrum
 
+    # Account for the existence of a mask.
+    if hasattr(calc_spectrum, 'mask') and calc_spectrum.mask is not None:
+        mask = calc_spectrum.mask
+        calc_spectrum = Spectrum1D(flux=calc_spectrum.flux[~mask],
+                              spectral_axis=calc_spectrum.spectral_axis[~mask])
+
     if continuum == 1:
         continuum = 1*calc_spectrum.flux.unit
 
-    spectral_axis = calc_spectrum.spectral_axis.quantity
+    spectral_axis = calc_spectrum.spectral_axis
 
-    # Ensure that the
-    dx = np.abs(spectral_axis[-1] - spectral_axis[0])
+    dx = (np.abs(spectral_axis[-1] - spectral_axis[0]))
 
-    line_flux = _compute_line_flux(spectrum, regions)
+    line_flux = _compute_line_flux(spectrum, regions,
+                                   mask_interpolation=mask_interpolation)
 
     # Calculate equivalent width
     ew = dx - (line_flux / continuum)
