@@ -2,7 +2,6 @@ import itertools
 import logging
 import operator
 
-import astropy.units as u
 import numpy as np
 from astropy.modeling import fitting, Model, models
 from astropy.table import QTable
@@ -10,7 +9,6 @@ from scipy.signal import convolve
 
 
 import astropy.units as u
-from astropy.stats import sigma_clipped_stats
 
 from ..spectra.spectral_region import SpectralRegion
 from ..spectra.spectrum1d import Spectrum1D
@@ -411,12 +409,10 @@ def _fit_lines(spectrum, model, fitter=fitting.LevMarLSQFitter(),
     mask = spectrum.mask
 
     dispersion = spectrum.spectral_axis
-    dispersion_unit = spectrum.spectral_axis.unit
 
     flux = spectrum.flux
     flux_unit = spectrum.flux.unit
 
-
     #
     # Determine the window if it is not None.  There
     # are several options here:
@@ -440,7 +436,7 @@ def _fit_lines(spectrum, model, fitter=fitting.LevMarLSQFitter(),
     # should fit
     elif window is not None and isinstance(window, tuple):
         window_indices = np.nonzero((dispersion >= window[0]) &
-                             (dispersion < window[1]))
+                                    (dispersion <= window[1]))
 
     # in this case the window is spectral regions that determine where
     # to fit.
@@ -483,7 +479,7 @@ def _fit_lines(spectrum, model, fitter=fitting.LevMarLSQFitter(),
 
     spectrum = Spectrum1D(
         flux=flux.value * flux_unit,
-        spectral_axis=dispersion.value * dispersion_unit,
+        spectral_axis=dispersion,
         wcs=input_spectrum.wcs,
         velocity_convention=input_spectrum.velocity_convention,
         rest_value=input_spectrum.rest_value)
