@@ -9,6 +9,7 @@ from gwcs.wcstools import grid_from_bounding_box
 
 from ...spectra import Spectrum1D, SpectrumList
 from ..registers import data_loader
+from ..parsing_utils import read_fileobj_or_hdulist
 
 
 __all__ = ["jwst_x1d_single_loader", "jwst_x1d_multi_loader"]
@@ -18,8 +19,8 @@ def identify_jwst_x1d_fits(origin, *args, **kwargs):
     """
     Check whether the given file is a JWST x1d spectral data product.
     """
-    is_jwst = _identify_jwst_fits(args[0])
-    with fits.open(args[0], memmap=False) as hdulist:
+    is_jwst = _identify_jwst_fits(*args)
+    with read_fileobj_or_hdulist(*args, memmap=False, **kwargs) as hdulist:
         return (is_jwst and 'EXTRACT1D' in hdulist and ('EXTRACT1D', 2) not in hdulist)
 
 
@@ -27,8 +28,8 @@ def identify_jwst_x1d_multi_fits(origin, *args, **kwargs):
     """
     Check whether the given file is a JWST x1d spectral data product with many slits.
     """
-    is_jwst = _identify_jwst_fits(args[0])
-    with fits.open(args[0], memmap=False) as hdulist:
+    is_jwst = _identify_jwst_fits(*args)
+    with read_fileobj_or_hdulist(*args, memmap=False, **kwargs) as hdulist:
         return is_jwst and ('EXTRACT1D', 2) in hdulist
 
 
@@ -36,47 +37,47 @@ def identify_jwst_s2d_fits(origin, *args, **kwargs):
     """
     Check whether the given file is a JWST s2d spectral data product.
     """
-    is_jwst = _identify_jwst_fits(args[0])
-    with fits.open(args[0], memmap=False) as hdulist:
+    is_jwst = _identify_jwst_fits(*args)
+    with read_fileobj_or_hdulist(*args, memmap=False, **kwargs) as hdulist:
         return (is_jwst and "SCI" in hdulist and ("SCI", 2) not in hdulist
-            and "EXTRACT1D" not in hdulist and len(hdulist["SCI"].data.shape) == 2)
+                and "EXTRACT1D" not in hdulist and len(hdulist["SCI"].data.shape) == 2)
 
 
 def identify_jwst_s2d_multi_fits(origin, *args, **kwargs):
     """
     Check whether the given file is a JWST s2d spectral data product with many slits.
     """
-    is_jwst = _identify_jwst_fits(args[0])
-    with fits.open(args[0], memmap=False) as hdulist:
+    is_jwst = _identify_jwst_fits(*args)
+    with read_fileobj_or_hdulist(*args, memmap=False, **kwargs) as hdulist:
         return (is_jwst and ("SCI", 2) in hdulist and "EXTRACT1D" not in hdulist
-            and len(hdulist["SCI"].data.shape) == 2)
+                and len(hdulist["SCI"].data.shape) == 2)
 
 
 def identify_jwst_s3d_fits(origin, *args, **kwargs):
     """
     Check whether the given file is a JWST s3d spectral data product.
     """
-    is_jwst = _identify_jwst_fits(args[0])
-    with fits.open(args[0], memmap=False) as hdulist:
+    is_jwst = _identify_jwst_fits(*args)
+    with read_fileobj_or_hdulist(*args, memmap=False, **kwargs) as hdulist:
         return (is_jwst and "SCI" in hdulist and "EXTRACT1D" not in hdulist
-            and len(hdulist["SCI"].data.shape) == 3)
+                and len(hdulist["SCI"].data.shape) == 3)
 
 
-def _identify_jwst_fits(filename):
+def _identify_jwst_fits(*args):
     """
     Check whether the given file is a JWST data product.
     """
     try:
-        with fits.open(filename, memmap=False) as hdulist:
-            return "ASDF" in hdulist and hdulist[0].header["TELESCOP"] == "JWST"
+        with read_fileobj_or_hdulist(*args, memmap=False) as hdulist:
+            return "ASDF" in hdulist and hdulist[0].header.get("TELESCOP") == "JWST"
     # This probably means we didn't have a FITS file
     except Exception:
         return False
 
 
 @data_loader("JWST x1d", identifier=identify_jwst_x1d_fits, dtype=Spectrum1D,
-            extensions=['fits'])
-def jwst_x1d_single_loader(filename, **kwargs):
+             extensions=['fits'])
+def jwst_x1d_single_loader(file_obj, **kwargs):
     """
     Loader for JWST x1d 1-D spectral data in FITS format
 
@@ -90,40 +91,42 @@ def jwst_x1d_single_loader(filename, **kwargs):
     Spectrum1D
         The spectrum contained in the file.
     """
-    spectrum_list = _jwst_x1d_loader(filename, **kwargs)
+    spectrum_list = _jwst_x1d_loader(file_obj, **kwargs)
     if len(spectrum_list) == 1:
         return spectrum_list[0]
     else:
         raise RuntimeError(f"Input data has {len(spectrum_list)} spectra. "
-            "Use SpectrumList.read() instead.")
+                           "Use SpectrumList.read() instead.")
 
 
 @data_loader("JWST x1d multi", identifier=identify_jwst_x1d_multi_fits,
-            dtype=SpectrumList, extensions=['fits'])
-def jwst_x1d_multi_loader(filename, **kwargs):
+             dtype=SpectrumList, extensions=['fits'])
+def jwst_x1d_multi_loader(file_obj, **kwargs):
     """
     Loader for JWST x1d 1-D spectral data in FITS format
 
     Parameters
     ----------
-    filename : str
-        The path to the FITS file
+    file_obj: str, file-like, or HDUList
+          FITS file name, object (provided from name by Astropy I/O Registry),
+          or HDUList (as resulting from astropy.io.fits.open()).
 
     Returns
     -------
     SpectrumList
         A list of the spectra that are contained in the file.
     """
-    return _jwst_x1d_loader(filename, **kwargs)
+    return _jwst_x1d_loader(file_obj, **kwargs)
 
 
-def _jwst_x1d_loader(filename, **kwargs):
+def _jwst_x1d_loader(file_obj, **kwargs):
     """Implementation of loader for JWST x1d 1-D spectral data in FITS format
 
     Parameters
     ----------
-    filename : str
-        The path to the FITS file
+    file_obj: str, file-like, or HDUList
+          FITS file name, object (provided from name by Astropy I/O Registry),
+          or HDUList (as resulting from astropy.io.fits.open()).
 
     Returns
     -------
@@ -133,7 +136,7 @@ def _jwst_x1d_loader(filename, **kwargs):
 
     spectra = []
 
-    with fits.open(filename, memmap=False) as hdulist:
+    with read_fileobj_or_hdulist(file_obj, memmap=False, **kwargs) as hdulist:
 
         primary_header = hdulist["PRIMARY"].header
 
@@ -163,24 +166,24 @@ def _jwst_x1d_loader(filename, **kwargs):
 
             else:
                 raise RuntimeError(f"Keyword SRCTYPE is {srctype}.  It should "
-                    "be 'POINT' or 'EXTENDED'. Can't decide between `flux` and "
-                    "`surf_bright` columns.")
+                                   "be 'POINT' or 'EXTENDED'. Can't decide between `flux` and "
+                                   "`surf_bright` columns.")
 
             # Merge primary and slit headers and dump into meta
             slit_header = hdu.header
             header = primary_header.copy()
             header.extend(slit_header, strip=True, update=True)
-            meta = {k: v for k,v in header.items()}
+            meta = {'header': header}
 
             spec = Spectrum1D(flux=flux, spectral_axis=wavelength,
-                uncertainty=uncertainty, meta=meta)
+                              uncertainty=uncertainty, meta=meta)
             spectra.append(spec)
 
     return SpectrumList(spectra)
 
 
 @data_loader("JWST s2d", identifier=identify_jwst_s2d_fits, dtype=Spectrum1D,
-            extensions=['fits'])
+             extensions=['fits'])
 def jwst_s2d_single_loader(filename, **kwargs):
     """
     Loader for JWST s2d 2D rectified spectral data in FITS format.
@@ -200,13 +203,13 @@ def jwst_s2d_single_loader(filename, **kwargs):
         return spectrum_list[0]
     elif len(spectrum_list) > 1:
         raise RuntimeError(f"Input data has {len(spectrum_list)} spectra. "
-            "Use SpectrumList.read() instead.")
+                           "Use SpectrumList.read() instead.")
     else:
         raise RuntimeError(f"Input data has {len(spectrum_list)} spectra.")
 
 
 @data_loader("JWST s2d multi", identifier=identify_jwst_s2d_multi_fits, dtype=SpectrumList,
-            extensions=['fits'])
+             extensions=['fits'])
 def jwst_s2d_multi_loader(filename, **kwargs):
     """
     Loader for JWST s2d 2D rectified spectral data in FITS format.
@@ -290,17 +293,17 @@ def _jwst_s2d_loader(filename, **kwargs):
                 # Make sure all rows are the same
                 if not (lam == wavelength_array).all():
                     raise RuntimeError("This 2D or 3D spectrum is not rectified "
-                        "and cannot be loaded into a Spectrum1D object.")
+                                       "and cannot be loaded into a Spectrum1D object.")
             elif dispaxis == 2:
                 flux_array = hdu.data.T
                 wavelength_array = lam[:, 0]
                 # Make sure all columns are the same
                 if not (lam.T == lam[None, :, 0]).all():
                     raise RuntimeError("This 2D or 3D spectrum is not rectified "
-                        "and cannot be loaded into a Spectrum1D object.")
+                                       "and cannot be loaded into a Spectrum1D object.")
             else:
                 raise RuntimeError("This 2D spectrum has an unknown dispaxis "
-                    "and cannot be loaded into a Spectrum1D object.")
+                                   "and cannot be loaded into a Spectrum1D object.")
 
             flux = Quantity(flux_array, unit=flux_unit)
             wavelength = Quantity(wavelength_array, unit=lam_unit)
@@ -309,7 +312,7 @@ def _jwst_s2d_loader(filename, **kwargs):
             slit_header = hdu.header
             header = primary_header.copy()
             header.extend(slit_header, strip=True, update=True)
-            meta = {k: v for k,v in header.items()}
+            meta = {'header': header}
 
             spec = Spectrum1D(flux=flux, spectral_axis=wavelength, meta=meta)
             spectra.append(spec)
@@ -318,7 +321,7 @@ def _jwst_s2d_loader(filename, **kwargs):
 
 
 @data_loader("JWST s3d", identifier=identify_jwst_s3d_fits, dtype=Spectrum1D,
-            extensions=['fits'])
+             extensions=['fits'])
 def jwst_s3d_single_loader(filename, **kwargs):
     """
     Loader for JWST s3d 3D rectified spectral data in FITS format.
@@ -338,7 +341,7 @@ def jwst_s3d_single_loader(filename, **kwargs):
         return spectrum_list[0]
     elif len(spectrum_list) > 1:
         raise RuntimeError(f"Input data has {len(spectrum_list)} spectra. "
-            "Use SpectrumList.read() instead.")
+                           "Use SpectrumList.read() instead.")
     else:
         raise RuntimeError(f"Input data has {len(spectrum_list)} spectra.")
 
@@ -393,7 +396,7 @@ def _jwst_s3d_loader(filename, **kwargs):
             slit_header = hdu.header
             header = primary_header.copy()
             header.extend(slit_header, strip=True, update=True)
-            meta = {k: v for k,v in header.items()}
+            meta = {'header': header}
 
             spec = Spectrum1D(flux=flux, spectral_axis=wavelength, meta=meta)
             spectra.append(spec)
