@@ -42,13 +42,16 @@ class SpectralRegion:
            The center of the spectral region.
 
         width : Scalar `~astropy.units.Quantity` with pixel or any valid ``spectral_axis`` unit
-           The width of the spectral region.
+           The full width of the spectral region (upper bound - lower bound).
         """
 
         if width.value <= 0:
             raise ValueError('SpectralRegion width must be positive.')
 
-        return cls(center - width, center + width)
+        if center.unit.physical_type != 'length':
+            return cls(center + width/2, center - width/2)
+
+        return cls(center - width/2, center + width/2)
 
     def __init__(self, *args):
         """
@@ -155,9 +158,10 @@ class SpectralRegion:
     def _valid(self):
 
         # Lower bound < Upper bound for all sub regions in length physical type
-        sub_regions = [(x[0].to('m'), x[1].to('m'))
-                       if x[0].unit.is_equivalent(u.m) else x
-                       for x in self._subregions]
+        with u.set_enabled_equivalencies(u.spectral()):
+            sub_regions = [(x[0].to('m'), x[1].to('m'))
+                           if x[0].unit.is_equivalent(u.m) else x
+                           for x in self._subregions]
 
         if any(x[0] >= x[1] for x in sub_regions):
             raise ValueError('Lower bound must be strictly less than the upper bound')
