@@ -3,32 +3,42 @@ from copy import deepcopy
 
 import numpy as np
 from astropy import units as u
-from astropy import constants as cnst
 from astropy.nddata import NDDataRef
 from astropy.utils.decorators import lazyproperty
 from .spectrum_mixin import OneDSpectrumMixin
-from .spectral_coordinate import SpectralCoord
+from .spectral_axis import SpectralAxis
 from ..utils.wcs_utils import gwcs_from_array
 
 __all__ = ['Spectrum1D']
 
 __doctest_skip__ = ['Spectrum1D.spectral_resolution']
 
-u.set_enabled_equivalencies(u.spectral())
-
 
 class Spectrum1D(OneDSpectrumMixin, NDDataRef):
     """
     Spectrum container for 1D spectral data.
 
+    Note that "1D" in this case refers to the fact that there is only one
+    spectral axis.  `Spectrum1D` can contain "vector 1D spectra" by having the
+    ``flux`` have a shape with dimension greater than 1.  The requirement
+    is that the last dimension of ``flux`` match the length of the
+    ``spectral_axis``.
+
+    For multidimensional spectra that are all the same shape but have different
+    spectral axes, use a :class:`~specutils.SpectrumCollection`.  For a
+    collection of spectra that have different shapes, use
+    :class:`~specutils.SpectrumList`. For more on this topic, see
+    :ref:`specutils-representation-overview`.
+
     Parameters
     ----------
-    flux : `astropy.units.Quantity` or astropy.nddata.NDData`-like
+    flux : `~astropy.units.Quantity` or `~astropy.nddata.NDData`-like
         The flux data for this spectrum.
-    spectral_axis : `astropy.units.Quantity` or `specutils.SpectralCoord`
+    spectral_axis : `~astropy.units.Quantity` or `~specutils.SpectralAxis`
         Dispersion information with the same shape as the last (or only)
-        dimension of flux.
-    wcs : `astropy.wcs.WCS` or `gwcs.wcs.WCS`
+        dimension of flux, or one greater than the last dimension of flux
+        if specifying bin edges.
+    wcs : `~astropy.wcs.WCS` or `~gwcs.wcs.WCS`
         WCS information object.
     velocity_convention : {"doppler_relativistic", "doppler_optical", "doppler_radio"}
         Convention used for velocity conversions.
@@ -40,17 +50,24 @@ class Spectrum1D(OneDSpectrumMixin, NDDataRef):
         See `redshift` for more information.
     radial_velocity
         See `radial_velocity` for more information.
+    bin_specification : str
+        Either "edges" or "centers" to indicate whether the `spectral_axis`
+        values represent edges of the wavelength bin, or centers of the bin.
     uncertainty : `~astropy.nddata.NDUncertainty`
         Contains uncertainty information along with propagation rules for
         spectrum arithmetic. Can take a unit, but if none is given, will use
         the unit defined in the flux.
+    mask : `~numpy.ndarray`-like
+        Array where values in the flux to be masked are those that
+        ``astype(bool)`` converts to True. (For example, integer arrays are not
+        masked where they are 0, and masked for any other value.)
     meta : dict
         Arbitrary container for any user-specific information to be carried
         around with the spectrum container object.
     """
     def __init__(self, flux=None, spectral_axis=None, wcs=None,
                  velocity_convention=None, rest_value=None, redshift=None,
-                 radial_velocity=None, **kwargs):
+                 radial_velocity=None, bin_specification=None, **kwargs):
         # Check for pre-defined entries in the kwargs dictionary.
         unknown_kwargs = set(kwargs).difference(
             {'data', 'unit', 'uncertainty', 'meta', 'mask', 'copy'})
@@ -62,9 +79,20 @@ class Spectrum1D(OneDSpectrumMixin, NDDataRef):
         # If the flux (data) argument is a subclass of nddataref (as it would
         # be for internal arithmetic operations), avoid setup entirely.
         if isinstance(flux, NDDataRef):
-            super(Spectrum1D, self).__init__(flux)
+            super().__init__(flux)
             return
 
+        # If the mask kwarg is not passed to the constructor, but the flux array
+        # contains NaNs, add the NaN locations to the mask.
+        if "mask" not in kwargs and flux is not None:
+            nan_mask = np.isnan(flux)
+            if nan_mask.any():
+                if hasattr(self, "mask"):
+                    kwargs["mask"] = np.logical_or(nan_mask, self.mask)
+                else:
+                    kwargs["mask"] = nan_mask.copy()
+            del nan_mask
+
         # Ensure that the flux argument is an astropy quantity
         if flux is not None:
             if not isinstance(flux, u.Quantity):
@@ -101,19 +129,42 @@ class Spectrum1D(OneDSpectrumMixin, NDDataRef):
                 rest_value = wcs.rest_frequency * u.Hz
             elif hasattr(wcs, 'rest_wavelength') and wcs.rest_wavelength != 0:
                 rest_value = wcs.rest_wavelength * u.AA
+            elif hasattr(wcs, 'wcs') and hasattr(wcs.wcs, 'restfrq') and wcs.wcs.restfrq > 0:
+                rest_value = wcs.wcs.restfrq * u.Hz
+            elif hasattr(wcs, 'wcs') and hasattr(wcs.wcs, 'restwav') and wcs.wcs.restwav > 0:
+                rest_value = wcs.wcs.restwav * u.m
             else:
-                rest_value = 0 * u.AA
+                rest_value = None
         else:
             if not isinstance(rest_value, u.Quantity):
                 logging.info("No unit information provided with rest value. "
                              "Assuming units of spectral axis ('%s').",
                              spectral_axis.unit)
                 rest_value = u.Quantity(rest_value, spectral_axis.unit)
-            elif not rest_value.unit.is_equivalent(u.AA) \
-                    and not rest_value.unit.is_equivalent(u.Hz):
+            elif not rest_value.unit.is_equivalent(u.AA, equivalencies=u.spectral()):
                 raise u.UnitsError("Rest value must be "
                                    "energy/wavelength/frequency equivalent.")
 
+        # If flux and spectral axis are both specified, check that their lengths
+        # match or are off by one (implying the spectral axis stores bin edges)
+        if flux is not None and spectral_axis is not None:
+            if spectral_axis.shape[0] == flux.shape[-1]:
+                if bin_specification == "edges":
+                    raise ValueError("A spectral axis input as bin edges"
+                        "must have length one greater than the flux axis")
+                bin_specification = "centers"
+            elif spectral_axis.shape[0] == flux.shape[-1]+1:
+                if bin_specification == "centers":
+                    raise ValueError("A spectral axis input as bin centers"
+                        "must be the same length as the flux axis")
+                bin_specification = "edges"
+            else:
+                raise ValueError(
+                    "Spectral axis length ({}) must be the same size or one "
+                    "greater (if specifying bin edges) than that of the last "
+                    "flux axis ({})".format(spectral_axis.shape[0], \
+                        flux.shape[-1]))
+
         # Attempt to parse the spectral axis. If none is given, try instead to
         # parse a given wcs. This is put into a GWCS object to
         # then be used behind-the-scenes for all specutils operations.
@@ -121,44 +172,42 @@ class Spectrum1D(OneDSpectrumMixin, NDDataRef):
             # Ensure that the spectral axis is an astropy Quantity
             if not isinstance(spectral_axis, u.Quantity):
                 raise ValueError("Spectral axis must be a `Quantity` or "
-                                 "`SpectralCoord` object.")
+                                 "`SpectralAxis` object.")
 
             # If spectral axis is provided as an astropy Quantity, convert it
-            # to a specutils SpectralCoord object.
-            if not isinstance(spectral_axis, SpectralCoord):
-                self._spectral_axis = SpectralCoord(
+            # to a specutils SpectralAxis object.
+            if not isinstance(spectral_axis, SpectralAxis):
+                if spectral_axis.shape[0] == flux.shape[-1] + 1:
+                    bin_specification = "edges"
+                else:
+                    bin_specification = "centers"
+                self._spectral_axis = SpectralAxis(
                     spectral_axis, redshift=redshift,
                     radial_velocity=radial_velocity, doppler_rest=rest_value,
-                    doppler_convention=velocity_convention)
-            # If a SpectralCoord object is provided, we assume it doesn't need
+                    doppler_convention=velocity_convention,
+                    bin_specification=bin_specification)
+            # If a SpectralAxis object is provided, we assume it doesn't need
             # information from other keywords added
             else:
                 for a in [radial_velocity, redshift]:
                     if a is not None:
                         raise ValueError("Cannot separately set redshift or "
-                                         "radial_velocity if a SpectralCoord "
+                                         "radial_velocity if a SpectralAxis "
                                          "object is input to spectral_axis")
 
                 self._spectral_axis = spectral_axis
 
-            wcs = gwcs_from_array(spectral_axis)
+            wcs = gwcs_from_array(self._spectral_axis)
         elif wcs is None:
             # If no spectral axis or wcs information is provided, initialize
             # with an empty gwcs based on the flux.
             size = len(flux) if not flux.isscalar else 1
             wcs = gwcs_from_array(np.arange(size) * u.Unit(""))
 
-        # Check to make sure the wavelength length is the same in both
-        if flux is not None and spectral_axis is not None:
-            if not spectral_axis.shape[0] == flux.shape[-1]:
-                raise ValueError(
-                    "Spectral axis ({}) and the last flux axis ({}) lengths "
-                    "must be the same.".format(
-                        spectral_axis.shape[0], flux.shape[-1]))
-
-        super(Spectrum1D, self).__init__(
+        super().__init__(
             data=flux.value if isinstance(flux, u.Quantity) else flux,
-            wcs=wcs, **kwargs)
+            wcs=wcs, **kwargs
+            )
 
         # If no spectral_axis was provided, create a SpectralCoord based on
         # the WCS
@@ -167,7 +216,10 @@ class Spectrum1D(OneDSpectrumMixin, NDDataRef):
             # the WCS
             spec_axis = self.wcs.pixel_to_world(np.arange(self.flux.shape[-1]))
 
-            self._spectral_axis = SpectralCoord(
+            if spec_axis.unit.is_equivalent(u.one):
+                spec_axis = spec_axis * u.pixel
+
+            self._spectral_axis = SpectralAxis(
                 spec_axis,
                 redshift=redshift, radial_velocity=radial_velocity,
                 doppler_rest=rest_value,
@@ -198,7 +250,8 @@ class Spectrum1D(OneDSpectrumMixin, NDDataRef):
             return self._copy(
                 flux=self.flux[item],
                 uncertainty=self.uncertainty[item]
-                if self.uncertainty is not None else None)
+                if self.uncertainty is not None else None,
+                mask=self.mask[item] if self.mask is not None else None)
 
         if not isinstance(item, slice):
             item = slice(item, item+1, None)
@@ -240,14 +293,14 @@ class Spectrum1D(OneDSpectrumMixin, NDDataRef):
     @property
     def frequency(self):
         """
-        The frequency as a `~astropy.units.Quantity` in units of GHz
+        The `spectral_axis` as a `~astropy.units.Quantity` in units of GHz
         """
         return self.spectral_axis.to(u.GHz, u.spectral())
 
     @property
     def wavelength(self):
         """
-        The wavelength as a `~astropy.units.Quantity` in units of Angstroms
+        The `spectral_axis` as a `~astropy.units.Quantity` in units of Angstroms
         """
         return self.spectral_axis.to(u.AA, u.spectral())
 
@@ -275,7 +328,7 @@ class Spectrum1D(OneDSpectrumMixin, NDDataRef):
 
     @lazyproperty
     def bin_edges(self):
-        return self.wcs.bin_edges()
+        return self.spectral_axis.bin_edges
 
     @property
     def shape(self):
