@@ -5,12 +5,21 @@ import urllib
 import tempfile
 
 
+remote_access = lambda argvals: pytest.mark.parametrize(
+    'remote_data_path', argvals, indirect=True, scope='function')
+
+
 @pytest.fixture(scope='module')
 def remote_data_path(request):
     """
     Remotely access the Zenodo deposition archive to retrieve the versioned
     test data.
     """
+    # Make use of configuration option from pytest-remotedata in order to
+    # control access to remote data.
+    if request.config.getoption('remote_data', 'any') != 'any':
+        pytest.skip()
+
     file_id, file_name = request.param.values()
 
     url = "https://zenodo.org/record/{}/files/{}?download=1".format(
@@ -24,4 +33,4 @@ def remote_data_path(request):
         with urllib.request.urlopen(url) as r, open(file_path, 'wb') as tmp_file:
             tmp_file.write(r.read())
 
-            yield file_path
\ No newline at end of file
+            yield file_path
