@@ -4,11 +4,12 @@ from astropy.modeling import models
 from astropy.nddata import StdDevUncertainty
 from astropy.tests.helper import assert_quantity_allclose
 
-from ..analysis import fwhm, centroid
-from ..fitting import (fit_lines, find_lines_derivative,
-                       find_lines_threshold, estimate_line_parameters)
-from ..manipulation import noise_region_uncertainty, spectrum_from_model
-from ..spectra import Spectrum1D, SpectralRegion
+from ..analysis import centroid, fwhm
+from ..fitting import (estimate_line_parameters, find_lines_derivative,
+                       find_lines_threshold, fit_lines)
+from ..manipulation import (extract_region, noise_region_uncertainty,
+                            spectrum_from_model)
+from ..spectra import SpectralRegion, Spectrum1D
 
 
 def single_peak():
@@ -225,7 +226,7 @@ def test_single_peak_fit_with_uncertainties():
             spec = Spectrum1D(spectral_axis=x, flux=y * u.Jy,
                               uncertainty=StdDevUncertainty(unc * u.Jy))
 
-            weights = 'unc' if implicit_weights else unc ** -2
+            weights = 'unc' if implicit_weights else unc ** -1
 
             spec_fit = fit_lines(spec, init_mod, weights=weights)
 
@@ -234,9 +235,9 @@ def test_single_peak_fit_with_uncertainties():
         return np.median(rms)
 
     assert np.allclose(calculate_rms(x, init_mod, implicit_weights=True),
-                       5.113708262419985)
+                       5.101611033799086)
     assert np.allclose(calculate_rms(x, init_mod, implicit_weights=False),
-                       5.147348340711497)
+                       5.113697654869089)
 
 
 def test_single_peak_fit_window():
@@ -442,11 +443,11 @@ def test_double_peak_fit_with_exclusion():
     y1_double_fit = g1_fit(x_double*u.um)
 
     # Comparing every 10th value.
-    y1_double_fit_expected = np.array([4.64465938e-130, 3.11793334e-103, 1.60765691e-079, 6.36698036e-059,
-                                       1.93681098e-041, 4.52537486e-027, 8.12148549e-016, 1.11951515e-007,
-                                       1.18532671e-002, 9.63961653e-001, 6.02136613e-002, 2.88897581e-006,
-                                       1.06464879e-013, 3.01357787e-024, 6.55197242e-038, 1.09414605e-054,
-                                       1.40343441e-074, 1.38268273e-097, 1.04632487e-123, 6.08168818e-153])
+    y1_double_fit_expected = np.array([1.22845792e-129, 6.72991033e-103, 2.89997996e-079, 9.82918347e-059,
+                                       2.62045444e-041, 5.49506092e-027, 9.06369193e-016, 1.17591109e-007,
+                                       1.19999812e-002, 9.63215290e-001, 6.08139060e-002, 3.02008492e-006,
+                                       1.17970152e-013, 3.62461471e-024, 8.75968088e-038, 1.66514266e-054,
+                                       2.48972475e-074, 2.92811381e-097, 2.70870241e-123, 1.97093074e-152])
 
     assert np.allclose(y1_double_fit.value[::10], y1_double_fit_expected, atol=1e-5)
 
@@ -678,3 +679,22 @@ def test_window_extras():
     weights = (~mask).astype(float)
     g_fit3 = fit_lines(s_msk, g_init, weights=weights, window=window_region)
     assert u.allclose(g_fit3.mean, 4.6, atol=.1)
+
+
+def test_fit_subspectrum():
+    # Create test spectrum
+    spectrum = Spectrum1D(
+        flux=np.random.sample(10) * u.Jy,
+        spectral_axis=(np.arange(10) + 6558) * u.AA,
+        rest_value=6561.16*u.angstrom,
+        velocity_convention='relativistic')
+
+    # Exract a sub region
+    sub_region = SpectralRegion(6490 * u.AA, 6635 * u.AA)
+    sub_spectrum = extract_region(spectrum, sub_region)
+
+    # Create model and git to spectrum
+    g_init = models.Gaussian1D(amplitude=1*u.Jy, mean=6561.157*u.angstrom,
+                                stddev=4.0721*u.angstrom)
+    g_fit = fit_lines(sub_spectrum, g_init)
+    y_fit = g_fit(sub_spectrum.spectral_axis)
