@@ -36,10 +36,10 @@ def test_region_simple(simulated_spectra):
 
 
 def test_region_ghz(simulated_spectra):
-    spectrum = Spectrum1D(flux=simulated_spectra.s1_um_mJy_e1,
+    spectrum = Spectrum1D(flux=simulated_spectra.s1_um_mJy_e1.flux,
                           spectral_axis=simulated_spectra.s1_um_mJy_e1.frequency)
 
-    region = SpectralRegion(374740.5725*u.GHz, 499654.09666667*u.GHz)
+    region = SpectralRegion(499654.09666667*u.GHz, 374740.5725*u.GHz)
 
     sub_spectrum = extract_region(spectrum, region)
 
@@ -175,8 +175,9 @@ def test_extract_region_mismatched_units():
 def test_linear_excise_invert_from_spectrum():
     spec = Spectrum1D(flux=np.random.sample(100) * u.Jy,
                       spectral_axis=np.arange(100) * u.AA)
-    inc_regs = SpectralRegion(80 * u.AA, 90 * u.AA) + \
-               SpectralRegion(50 * u.AA, 60 * u.AA)
+    inc_regs = SpectralRegion(0 * u.AA, 50 * u.AA) + \
+               SpectralRegion(60 * u.AA, 80 * u.AA) + \
+               SpectralRegion(90 * u.AA, 110 * u.AA)
     exc_regs = inc_regs.invert_from_spectrum(spec)
 
     excised_spec = linear_exciser(spec, exc_regs)
