@@ -7,8 +7,7 @@ from astropy.nddata import StdDevUncertainty
 
 from .conftest import remote_access
 from ..spectra import Spectrum1D
-from ..spectra.spectral_coordinate import SpectralCoord
-
+from ..extern.spectralcoord import SpectralCoord
 
 def test_empty_spectrum():
     spec = Spectrum1D(spectral_axis=[]*u.um,
@@ -51,9 +50,10 @@ def test_create_from_multidimensional_arrays():
     assert (spec.frequency == freqs).all()
     assert (spec.flux == flux).all()
 
-    # Mis-matched lengths should raise an exception
+    # Mis-matched lengths should raise an exception (unless freqs is one longer
+    # than flux, in which case it's interpreted as bin edges)
     freqs = np.arange(50) * u.GHz
-    flux = np.random.random((5, len(freqs)-1)) * u.Jy
+    flux = np.random.random((5, len(freqs)-10)) * u.Jy
     with pytest.raises(ValueError) as e_info:
         spec = Spectrum1D(spectral_axis=freqs, flux=flux)
 
@@ -65,10 +65,11 @@ def test_create_from_quantities():
     assert spec.spectral_axis.unit == u.nm
     assert spec.spectral_axis.size == 49
 
-    # Mis-matched lengths should raise an exception
+    # Mis-matched lengths should raise an exception (unless freqs is one longer
+    # than flux, in which case it's interpreted as bin edges)
     with pytest.raises(ValueError) as e_info:
         spec = Spectrum1D(spectral_axis=np.arange(1, 50) * u.nm,
-                      flux=np.random.randn(48) * u.Jy)
+                      flux=np.random.randn(47) * u.Jy)
 
 
 def test_create_implicit_wcs():
@@ -133,6 +134,17 @@ def test_spectral_axis_conversions():
     new_spec = spec.with_spectral_unit(u.GHz)
 
 
+@pytest.mark.parametrize('unit', ['micron', 'GHz', 'cm**-1', 'eV'])
+def test_spectral_axis_equivalencies(unit):
+    """Test that `u.spectral` equivalencies are enabled for `spectral_axis`."""
+
+    spectral_axis=np.array([3400, 5000, 6660]) * u.AA
+    spec = Spectrum1D(flux=np.array([26.0, 30.0, 44.5]) * u.Jy, spectral_axis=spectral_axis)
+
+    new_axis = spectral_axis.to(unit, equivalencies=u.spectral())
+    assert u.allclose(spec.spectral_axis.to(unit), new_axis)
+
+
 def test_redshift():
     spec = Spectrum1D(flux=np.array([26.0, 30., 44.5]) * u.Jy,
                       spectral_axis=np.array([4000, 6000, 8000]) * u.AA,
@@ -148,8 +160,8 @@ def test_redshift():
                       rest_value=6000 * u.AA,
                       redshift= 0.1)
 
-    assert u.allclose(spec.velocity, [-69951.3, 29979.2, 129910.1]*u.km/u.s,
-                     atol=0.5*u.km/u.s)
+    assert u.allclose(spec.velocity, [-71443.75318854, 28487.0661448, 128417.88547813]*u.km/u.s,
+                      atol=0.5*u.km/u.s)
 
     #-------------------------
 
@@ -159,7 +171,7 @@ def test_redshift():
                       rest_value=11.0 * u.GHz)
 
     assert u.allclose(spec.velocity, [13626., 0, -13626]*u.km/u.s,
-                     atol=1*u.km/u.s)
+                      atol=1*u.km/u.s)
 
     spec = Spectrum1D(flux=np.array([26.0, 30.0, 44.5]) * u.Jy,
                       spectral_axis=np.array([10.5, 11.0, 11.5]) * u.GHz,
@@ -167,7 +179,7 @@ def test_redshift():
                       rest_value=11.0 * u.GHz,
                       redshift= 0.1)
 
-    assert u.allclose(spec.velocity, [43606., 29979., 16352.]*u.km/u.s,
+    assert u.allclose(spec.velocity, [42113.99605389, 28487.0661448 , 14860.13623571]*u.km/u.s,
                       atol=1*u.km/u.s)
 
     #------------------------- radial velocity mode
@@ -323,12 +335,22 @@ def test_energy_photon_flux():
     assert spec.photon_flux.unit == u.photon * u.cm**-2 * u.s**-1 * u.nm**-1
 
 
+def test_flux_nans_propagate_to_mask():
+    """Check that indices in input flux with NaNs get propagated to the mask"""
+    flux = np.random.randn(10)
+    nan_idx = [0, 3, 5]
+    flux[nan_idx] = np.nan
+    spec = Spectrum1D(spectral_axis=np.linspace(100, 1000, 10) * u.nm,
+                      flux=flux * u.Jy)
+    assert spec.mask[nan_idx].all() == True
+
+
 def test_repr():
     spec_with_wcs = Spectrum1D(spectral_axis=np.linspace(100, 1000, 10) * u.nm,
                                flux=np.random.random(10) * u.Jy)
     result = repr(spec_with_wcs)
     assert result.startswith('<Spectrum1D(flux=<Quantity [')
-    assert 'spectral_axis=<SpectralCoord [' in result
+    assert 'spectral_axis=<SpectralAxis' in result
 
     spec_with_unc = Spectrum1D(spectral_axis=np.linspace(100, 1000, 10) * u.nm,
                                flux=np.random.random(10) * u.Jy,
@@ -336,7 +358,7 @@ def test_repr():
                                    np.random.sample(10), unit='Jy'))
     result = repr(spec_with_unc)
     assert result.startswith('<Spectrum1D(flux=<Quantity [')
-    assert 'spectral_axis=<SpectralCoord [' in result
+    assert 'spectral_axis=<SpectralAxis' in result
     assert 'uncertainty=StdDevUncertainty(' in result
 
 
@@ -385,3 +407,14 @@ def test_str():
 """Spectrum1D (length=1)
 flux:             [ 1.0 Jy ],  mean=1.0 Jy
 spectral axis:    [ 0.0 nm ],  mean=0.0 nm"""
+
+
+def test_equivalencies():
+    """
+    Test that after import `u.spectral` equivalencies are not enabled in the global namespace.
+    """
+    assert u.micron.is_equivalent(u.cm**-1) is False
+    assert u.micron.is_equivalent(u.Hz) is False
+    assert u.micron.is_equivalent(u.eV) is False
+    assert u.Hz.is_equivalent(u.cm**-1) is False
+    assert u.Hz.is_equivalent(u.eV) is False
